/*
 * =====================================================================================
 * 
 *     Modulename:
 *       Filename:  watchface_double_time_zone.c
 *
 *    Description:  双时区表盘
 *    Corporation:
 * 
 *         Author:  gliu (), gliu@maibu.cc
 *        Created:  2015年03月27日 10时52分52秒
 *
 * =====================================================================================
 *
 * =====================================================================================
 * 
 *   MODIFICATION HISTORY :
 *    
 *		     DATE :
 *		     DESC :
 * =====================================================================================
 */	
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#ifdef LINUX
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include "linux_screen_show.h"
#include "os_store_manage.h"
#include "os_time.h"
#include "os_comm.h"
#include "os_res_bitmap.h"
#include "os_sys_app_id.h"
#include "os_memory_manage.h"
#include "res_user_bitmap.h"
#include "os_sys_event.h"
#include "window_stack.h"
#include "window.h"
#include "screen_show.h"
#include "matrix.h"
#include "plug_status_bar.h"
#include "res_bitmap_base.h"
#include "os_user_app_id.h"
#include "json_checker.h"
#else
#include "maibu_sdk.h"
#include "maibu_res.h"
#endif


typedef struct tag_DoubleTimeZone
{
	int8_t flag;		/*是否使用标志*/
	char zone1_addr[25];	/*地域名1*/
	char zone2_addr[25];	/*地域名2*/
	int8_t  zone1_offset;	/*地域1时区与手机所在时区差别,单位小时*/	
	int8_t  zone2_offset;	/*地域1时区与手机所在时区差别,单位小时*/	
}DoubleTimeZone;


/*全局变量保存数据*/
static DoubleTimeZone  g_double_timezone = {0,"","",0,0};


/*窗口ID, 通过该窗口ID获取窗口句柄*/
static int32_t g_double_timezone_window_id = -1;

/*图层ID，通过该图层ID获取图层句柄*/

/*定义各个图层的位置*/

/*背景图层*/
#define TIMEZONE_BG_ORIGIN_X			0
#define TIMEZONE_BG_ORIGIN_Y			0
#define TIMEZONE_BG_SIZE_H			128
#define TIMEZONE_BG_SIZE_W			128


/*地域1图层*/
#define TIMEZONE_ADDR1_ORIGIN_X			4
#define TIMEZONE_ADDR1_ORIGIN_Y			8
#define TIMEZONE_ADDR1_SIZE_H			12
#define TIMEZONE_ADDR1_SIZE_W			120


/*地域2图层*/
#define TIMEZONE_ADDR2_ORIGIN_X		4
#define TIMEZONE_ADDR2_ORIGIN_Y		65
#define TIMEZONE_ADDR2_SIZE_H		12
#define TIMEZONE_ADDR2_SIZE_W		120


/*差距图层*/
#define TIMEZONE_OFFSET_ORIGIN_X		4
#define TIMEZONE_OFFSET_ORIGIN_Y		114	
#define TIMEZONE_OFFSET_SIZE_H		12
#define TIMEZONE_OFFSET_SIZE_W		120

/*地域1时间图层*/
#define TIMEZONE_TIME1_ORIGIN_X		4
#define TIMEZONE_TIME1_ORIGIN_Y		26
#define TIMEZONE_TIME1_SIZE_H		23
#define TIMEZONE_TIME1_SIZE_W		84

/*地域2时间图层*/
#define TIMEZONE_TIME2_ORIGIN_X		4
#define TIMEZONE_TIME2_ORIGIN_Y		83
#define TIMEZONE_TIME2_SIZE_H		23
#define TIMEZONE_TIME2_SIZE_W		84



/*保存配置信息文件KEY*/
#define DOUBLE_TIMEZONE_KEY	1





P_Window double_timezone_create_window()
{
	P_Window p_window = app_window_create();
	if (NULL == p_window)
	{
		return NULL;
	}

	GBitmap bmp;
	
	/*创建背景图层*/
	GRect frame_bg = {{TIMEZONE_BG_ORIGIN_X, TIMEZONE_BG_ORIGIN_Y}, {TIMEZONE_BG_SIZE_H, TIMEZONE_BG_SIZE_W}};
	res_get_user_bitmap(RES_BITMAP_WATCHFACE_TIMEZONE_BG, &bmp);			
	LayerBitmap layer_bg_struct = {bmp, frame_bg, GAlignCenter};
	P_Layer layer_bg = app_layer_create_bitmap(&layer_bg_struct);
	app_window_add_layer(p_window, layer_bg);


	struct date_time t;
	app_service_get_datetime(&t);
	int32_t bmp_id[10] = {RES_BITMAP_WATCHFACE_TIMEZONE_0, RES_BITMAP_WATCHFACE_TIMEZONE_1,RES_BITMAP_WATCHFACE_TIMEZONE_2, RES_BITMAP_WATCHFACE_TIMEZONE_3,
		RES_BITMAP_WATCHFACE_TIMEZONE_4, RES_BITMAP_WATCHFACE_TIMEZONE_5, RES_BITMAP_WATCHFACE_TIMEZONE_6, RES_BITMAP_WATCHFACE_TIMEZONE_7,
		RES_BITMAP_WATCHFACE_TIMEZONE_8, RES_BITMAP_WATCHFACE_TIMEZONE_9};

	uint8_t zone_offset;


	/*创建时区1地址*/
	GRect frame_addr1 = {{TIMEZONE_ADDR1_ORIGIN_X, TIMEZONE_ADDR1_ORIGIN_Y}, {TIMEZONE_ADDR1_SIZE_H, TIMEZONE_ADDR1_SIZE_W}};
	LayerText layer_addr1_struct = {g_double_timezone.zone1_addr, frame_addr1, GAlignLeft, U_GBK_SIMSUN_12};
	P_Layer layer_addr1 = app_layer_create_text(&layer_addr1_struct);
	app_window_add_layer(p_window, layer_addr1);


	/*创建时1图层*/
	res_get_user_bitmap(bmp_id[(t.hour+g_double_timezone.zone1_offset)/10], &bmp);
	GRect frame_zone1_h1 = {{TIMEZONE_TIME1_ORIGIN_X, TIMEZONE_TIME1_ORIGIN_Y}, {TIMEZONE_TIME1_SIZE_H, bmp.width}};
	LayerBitmap layer_zone1_h1_struct = {bmp, frame_zone1_h1, GAlignCenter};
	P_Layer layer_zone1_h1 = app_layer_create_bitmap(&layer_zone1_h1_struct);
	app_layer_set_bg_color(layer_zone1_h1, GColorBlack);
	app_window_add_layer(p_window, layer_zone1_h1);
	zone_offset = TIMEZONE_TIME1_ORIGIN_X+bmp.width;	


	/*创建时2图层*/
	res_get_user_bitmap(bmp_id[(t.hour+g_double_timezone.zone1_offset)%10], &bmp);
	GRect frame_zone1_h2 = {{zone_offset, TIMEZONE_TIME1_ORIGIN_Y}, {TIMEZONE_TIME1_SIZE_H, bmp.width}};
	LayerBitmap layer_zone1_h2_struct = {bmp, frame_zone1_h2, GAlignCenter};
	P_Layer layer_zone1_h2 = app_layer_create_bitmap(&layer_zone1_h2_struct);
	app_layer_set_bg_color(layer_zone1_h2, GColorBlack);
	app_window_add_layer(p_window, layer_zone1_h2);
	zone_offset += bmp.width;

	/*创建冒号图层*/
	res_get_user_bitmap(RES_BITMAP_WATCHFACE_TIMEZONE_MAOHAO, &bmp);
	GRect frame_zone1_maohao = {{zone_offset, TIMEZONE_TIME1_ORIGIN_Y}, {TIMEZONE_TIME1_SIZE_H, bmp.width}};
	LayerBitmap layer_zone1_maohao_struct = {bmp, frame_zone1_maohao, GAlignCenter};
	P_Layer layer_zone1_maohao = app_layer_create_bitmap(&layer_zone1_maohao_struct);
	app_layer_set_bg_color(layer_zone1_maohao, GColorBlack);
	app_window_add_layer(p_window, layer_zone1_maohao);
	zone_offset += bmp.width;


	/*创建分1图层*/
	res_get_user_bitmap(bmp_id[t.min/10], &bmp);
	GRect frame_zone1_m1 = {{zone_offset, TIMEZONE_TIME1_ORIGIN_Y}, {TIMEZONE_TIME1_SIZE_H, bmp.width}};
	LayerBitmap layer_zone1_m1_struct = {bmp, frame_zone1_m1, GAlignCenter};
	P_Layer layer_zone1_m1 = app_layer_create_bitmap(&layer_zone1_m1_struct);
	app_layer_set_bg_color(layer_zone1_m1, GColorBlack);
	app_window_add_layer(p_window, layer_zone1_m1);
	zone_offset += bmp.width;


	/*创建分2图层*/
	res_get_user_bitmap(bmp_id[t.min%10], &bmp);
	GRect frame_zone1_m2 = {{zone_offset, TIMEZONE_TIME1_ORIGIN_Y}, {TIMEZONE_TIME1_SIZE_H, bmp.width}};
	LayerBitmap layer_zone1_m2_struct = {bmp, frame_zone1_m2, GAlignCenter};
	P_Layer layer_zone1_m2 = app_layer_create_bitmap(&layer_zone1_m2_struct);
	app_layer_set_bg_color(layer_zone1_m2, GColorBlack);
	app_window_add_layer(p_window, layer_zone1_m2);



	/*创建时区2地址*/
	GRect frame_addr2 = {{TIMEZONE_ADDR2_ORIGIN_X, TIMEZONE_ADDR2_ORIGIN_Y}, {TIMEZONE_ADDR2_SIZE_H, TIMEZONE_ADDR2_SIZE_W}};
	LayerText layer_addr2_struct = {g_double_timezone.zone2_addr, frame_addr2, GAlignLeft, U_GBK_SIMSUN_12};
	P_Layer layer_addr2 = app_layer_create_text(&layer_addr2_struct);
	app_layer_set_bg_color(layer_addr2, GColorBlack);
	app_window_add_layer(p_window, layer_addr2);


	/*创建时1图层*/
	res_get_user_bitmap(bmp_id[(t.hour+g_double_timezone.zone2_offset)/10], &bmp);
	GRect frame_zone2_h1 = {{TIMEZONE_TIME2_ORIGIN_X, TIMEZONE_TIME2_ORIGIN_Y}, {TIMEZONE_TIME2_SIZE_H, bmp.width}};
	LayerBitmap layer_zone2_h1_struct = {bmp, frame_zone2_h1, GAlignCenter};
	P_Layer layer_zone2_h1 = app_layer_create_bitmap(&layer_zone2_h1_struct);
	app_window_add_layer(p_window, layer_zone2_h1);
	zone_offset = TIMEZONE_TIME2_ORIGIN_X+bmp.width;	


	/*创建时2图层*/
	res_get_user_bitmap(bmp_id[(t.hour+g_double_timezone.zone2_offset)%10], &bmp);
	GRect frame_zone2_h2 = {{zone_offset, TIMEZONE_TIME2_ORIGIN_Y}, {TIMEZONE_TIME2_SIZE_H, bmp.width}};
	LayerBitmap layer_zone2_h2_struct = {bmp, frame_zone2_h2, GAlignCenter};
	P_Layer layer_zone2_h2 = app_layer_create_bitmap(&layer_zone2_h2_struct);
	app_window_add_layer(p_window, layer_zone2_h2);
	zone_offset += bmp.width;

	/*创建冒号图层*/
	res_get_user_bitmap(RES_BITMAP_WATCHFACE_TIMEZONE_MAOHAO, &bmp);
	GRect frame_zone2_maohao = {{zone_offset, TIMEZONE_TIME2_ORIGIN_Y}, {TIMEZONE_TIME2_SIZE_H, bmp.width}};
	LayerBitmap layer_zone2_maohao_struct = {bmp, frame_zone2_maohao, GAlignCenter};
	P_Layer layer_zone2_maohao = app_layer_create_bitmap(&layer_zone2_maohao_struct);
	app_window_add_layer(p_window, layer_zone2_maohao);
	zone_offset += bmp.width;


	/*创建分1图层*/
	res_get_user_bitmap(bmp_id[t.min/10], &bmp);
	GRect frame_zone2_m1 = {{zone_offset, TIMEZONE_TIME2_ORIGIN_Y}, {TIMEZONE_TIME2_SIZE_H, bmp.width}};
	LayerBitmap layer_zone2_m1_struct = {bmp, frame_zone2_m1, GAlignCenter};
	P_Layer layer_zone2_m1 = app_layer_create_bitmap(&layer_zone2_m1_struct);
	app_window_add_layer(p_window, layer_zone2_m1);
	zone_offset += bmp.width;


	/*创建分2图层*/
	res_get_user_bitmap(bmp_id[t.min%10], &bmp);
	GRect frame_zone2_m2 = {{zone_offset, TIMEZONE_TIME2_ORIGIN_Y}, {TIMEZONE_TIME2_SIZE_H, bmp.width}};
	LayerBitmap layer_zone2_m2_struct = {bmp, frame_zone2_m2, GAlignCenter};
	P_Layer layer_zone2_m2 = app_layer_create_bitmap(&layer_zone2_m2_struct);
	app_window_add_layer(p_window, layer_zone2_m2);

	
	/*创建时差图层*/
	GRect frame_offset = {{TIMEZONE_OFFSET_ORIGIN_X, TIMEZONE_OFFSET_ORIGIN_Y}, {TIMEZONE_OFFSET_SIZE_H, TIMEZONE_OFFSET_SIZE_W}};
	char str_tmp[20] = "";
	if (g_double_timezone.zone1_offset >= g_double_timezone.zone2_offset)
	{
		sprintf(str_tmp, "晚%d小时", g_double_timezone.zone1_offset - g_double_timezone.zone2_offset);	
	}
	else
	{
		sprintf(str_tmp, "早%d小时", g_double_timezone.zone2_offset - g_double_timezone.zone1_offset);	
	}
	LayerText layer_offset_struct = {str_tmp, frame_offset, GAlignLeft, U_GBK_SIMSUN_12};
	P_Layer layer_offset = app_layer_create_text(&layer_offset_struct);
	app_layer_set_bg_color(layer_offset, GColorBlack);
	app_window_add_layer(p_window, layer_offset);

	return (p_window);
}



void app_double_timezone_watch_time_change(enum SysEventType type, void *context)
{
	/*时间更改*/
	if (type == SysEventTypeTimeChange)
	{
		/*根据窗口ID获取窗口句柄*/
		P_Window p_old_window = app_window_stack_get_window_by_id(g_double_timezone_window_id);	
		if (NULL == p_old_window)
		{
			return;	
		}

		P_Window p_new_window = double_timezone_create_window();
		if (NULL != p_new_window)
		{
			g_double_timezone_window_id = app_window_stack_replace_window(p_old_window, p_new_window);
		}	

	}

	return;
}




P_Window double_timezone_init()
{
	/*全局变量是否有值*/
	if (g_double_timezone.flag == 0)
	{
		/*创建一个可读可写的保存构体的文件key*/
		#ifdef LINUX
		app_persist_create(USER_APP_ID_DOUBLE_TIMEZONE, DOUBLE_TIMEZONE_KEY, sizeof(DoubleTimeZone));
		#else
		app_persist_create(DOUBLE_TIMEZONE_KEY, sizeof(DoubleTimeZone));
		#endif

		/*读取结构信息*/
		#ifdef LINUX
		app_persist_read_data(USER_APP_ID_DOUBLE_TIMEZONE, DOUBLE_TIMEZONE_KEY, 0, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
		#else
		app_persist_read_data(DOUBLE_TIMEZONE_KEY, 0, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
		#endif
		
		if (g_double_timezone.flag == 0)
		{
			strcpy(g_double_timezone.zone1_addr, "北京");
			strcpy(g_double_timezone.zone2_addr, "巴塞罗那");
			g_double_timezone.zone1_offset = 0;
			g_double_timezone.zone2_offset = -7;	
			g_double_timezone.flag = 1;
			
			/*保存数据*/
			#ifdef LINUX	
			app_persist_write_data_extend( USER_APP_ID_DOUBLE_TIMEZONE, DOUBLE_TIMEZONE_KEY, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
			#else
			app_persist_write_data_extend( DOUBLE_TIMEZONE_KEY, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
			#endif
			
		}	

	}


	P_Window p_window = double_timezone_create_window();

	/*注册一个事件通知回调，当有时间改变是，立即更新时间*/
	maibu_service_sys_event_subscribe(app_double_timezone_watch_time_change);

	return p_window;

}

void double_timezone_param_callback(enum ESyncWatchApp type, uint8_t *context, uint16_t context_size)
{
	if(type == ESyncWatchAppUpdateParam)
	{
		/*如果配置参数不为空，则是手机客户端下发配置信息，需要保存*/
		if (context != NULL)
		{
			/*json数据格式为{"zone1":"纽约","offset1":"8","zone2":"北京","offset2":"0"}*/			
			DoubleTimeZone data;
			memset(&data, 0, sizeof(DoubleTimeZone));
			int32_t offset1 = 0, offset2 = 0;
			if((0 == maibu_get_json_str((const char *)context, "zone1", data.zone1_addr, sizeof(data.zone1_addr))) 
				&& (0 == maibu_get_json_str((const char *)context, "zone2", data.zone2_addr, sizeof(data.zone2_addr)))
				&& (0 == maibu_get_json_int((const char *)context, "offset1", &offset1))
				&& (0 == maibu_get_json_int((const char *)context, "offset2", &offset2)))
			{
				data.zone1_offset = offset1;
				data.zone2_offset = offset2;
				g_double_timezone = data;

				/*保存配置*/
				#ifdef LINUX	
				app_persist_write_data_extend( USER_APP_ID_DOUBLE_TIMEZONE, DOUBLE_TIMEZONE_KEY, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
				#else
				app_persist_write_data_extend( DOUBLE_TIMEZONE_KEY, (unsigned char *)&g_double_timezone, sizeof(DoubleTimeZone));
				#endif

			}
		}
	
	}
	else if(type == ESyncWatchAppGetParam)
	{
		char json[100] =  "";	//填充的数据为字符串
		sprintf(json, "{\"zone1\":\"%s\",\"offset1\":\"%d\",\"zone2\":\"%s\",\"offset2\":\"%d\"}", g_double_timezone.zone1_addr, g_double_timezone.zone1_offset,
			g_double_timezone.zone2_addr, g_double_timezone.zone2_offset);

		maibu_comm_sync_watchapp(ESyncWatchAppUpdateParam, (uint8_t *)json, strlen(json));	

	}


}


int main()
{

#ifdef LINUX	
	/*非APP编写*/	
	screen_init(SCREEN_ROW_NUMS,SCREEN_COL_NUMS);
	os_store_manage_init();	
	window_stack_init();
	set_current_app(0x7dd01);
	SHOW;
#endif


	/*创建显示汉字表盘窗口*/
	P_Window p_window = double_timezone_init(); 
	if(p_window != NULL)
	{
		/*放入窗口栈显示*/
		g_double_timezone_window_id = app_window_stack_push(p_window);

		/*创建接受配置信息回调*/
		maibu_comm_register_watchapp_callback(double_timezone_param_callback);	
	}


#ifdef LINUX
	SHOW;
	/*非APP编写*/	
	while (1)
	{
		char input;	
	
		/*输入操作*/
		scanf("%c", &input);
		if (input == 'q')
		{
			break;	
		}
		else if (input == 'c')
		{
	
		}
		else if (input == 'g')
		{
			break;
		}
		else if(input == 's')
		{
		//	double_timezone_param_callback(CommRequestAppParam, NULL);
		}	
	}	

	app_window_stack_pop(p_window);
	window_stack_destory();
	screen_destory();
	os_store_manage_destory();

	SHOW;

#endif

	return 0;

}


